function THR = thselect(X,TPTR)

//thselect is a one-dimensional de-noising oriented function.

//THR = thselect(X,TPTR) returns threshold X-adapted value using selection rule defined by string TPTR.

//Available selection rules are

//    * TPTR = 'rigrsure', adaptive threshold selection using principle of Stein's Unbiased Risk Estimate.
//    * TPTR = 'heursure', heuristic variant of the first option.
//    * TPTR = 'sqtwolog', threshold is sqrt(2*log(length(X))).
//    * TPTR = 'minimaxi', minimax thresholding.

  [nargout,nargin]=argn(0);
  if (nargin < 2),
	error ( '2 parameters are required' );
  end

 if ~(convstr(TPTR)=='rigrsure' | convstr(TPTR)=='heursure' | convstr(TPTR)=='sqtwolog' | convstr(TPTR)=='minimaxi')
	    error("TPTR must be rigrsure, heursure, sqtwolog or minimaxi!");
 end;

  select convstr(TPTR),
    case 'rigrsure'
      THR  = ValSUREThresh(X);
    case 'heursure'
	[n,J] = dyadlength(X);
	magic = sqrt(2*log(n));
	eta = (norm(X).^2 - n)/n;
	crit = J^(1.5)/sqrt(n);
	if eta < crit
		THR=magic;
	else
		THR  = min(ValSUREThresh(X), magic);
	end
    case 'sqtwolog'
	THR=sqrt(2*log(length(X)));
    case 'minimaxi'
	lamlist = [0  0 0 0 0 1.27 1.47 1.67 1.86 2.05 2.23 2.41 2.6 2.77 2.95 3.13 3.31 3.49 3.67 3.85 4.03 4.21];
	[n,j] = dyadlength(X);
	THR = lamlist(j);
   else
      error("wrong parameter for threshhold selection!");
  end

endfunction


function thresh = ValSUREThresh(x)
// ValSUREThresh -- Adaptive Threshold Selection Using Principle of SURE
//  Usage
//    thresh = ValSUREThresh(y)
//  Inputs
//    y        Noisy Data with Std. Deviation = 1
//  Outputs
//    thresh   Value of Threshold
//
//  Description
//    SURE referes to Stein's Unbiased Risk Estimate.
//
	x=x(:);
	a = mtlb_sort(abs(x)).^2 ;
	b = cumsum(a,'m');
	n = length(x);
	c = linspace(n-1,0,n);
	s = b+c(:).*a;
	risk = (n - ( 2 .* (1:n ))  + s')'/n;
	[guess,ibest] = min(risk,'m');
	thresh = sqrt(a(ibest));

//
// Copyright (c) 1993-5.  Jonathan Buckheit, David Donoho and Iain Johnstone
//

endfunction

function [n,J] = dyadlength(x)
// dyadlength -- Find length and dyadic length of array
//  Usage
//    [n,J] = dyadlength(x)
//  Inputs
//    x    array of length n = 2^J (hopefully)
//  Outputs
//    n    length(x)
//    J    least power of two greater than n
//
//  Side Effects
//    A warning is issued if n is not a power of 2.
//
//  See Also
//    quadlength, dyad, dyad2ix
//
  n = length(x) ;
  J = ceil(log(n)/log(2));
  if 2^J ~= n ,
      disp('Warning in dyadlength: n != 2^J')
  end

    endfunction

